// vim: set sw=4 ts=4 et: 
// IPv6 calculator function, License: MIT, Copyright: Guido Socher

// syntax check IPv6 address
function isValidIpV6(ipv6str){
    if (/^[0-9A-Fa-f:]+:\d+\.\d+\.\d+\.\d+$/.test(ipv6str)){
        // allow ipv4 mapped addresses
        var countArr=ipv6str.match(/:/g);
        if (! countArr){
            return(false);
        }
        // 32 bit are taken by the dotted decimal notation:
        if (countArr.length <2 || countArr.length > 6){
            return(false);
        }
        return(true);
    }
    if (! /^[0-9A-Fa-f:]+$/.test(ipv6str)){
        return(false);
    }
    var countArr=ipv6str.match(/:/g);
    if (! countArr){
        return(false);
    }
    if (countArr.length <2 || countArr.length > 7){
        return(false);
    }
    var numOfDoubleColonArr=ipv6str.match(/::/g);
    // double colon is only allowed once in the string
    if (numOfDoubleColonArr){
        if(numOfDoubleColonArr.length!=1){
            return(false);
        }
    }else{
        // no double colon, we must have 8 fields
        if (countArr.length!=7){
            return(false);
        }
    }
    if (ipv6str.match(/^:\w/)){
        // empty or trailing fields have to be before or after :: (::1, ::, 1:: are all valid but :1: is invalid)
        return(false);
    }
    if (ipv6str.match(/\w:$/)){
        // empty or trailing fields have to be before or after :: (::1, ::, 1:: are all valid but :1: is invalid)
        return(false);
    }
    var ipFieldArr = ipv6str.split(":");
    // ipFieldArr[0] is the left most field in the ipv6 addr
    var i=0;
    var ipFieldArrL=ipFieldArr.length;
    while (i<ipFieldArrL){
        if (ipFieldArr[i].length > 4){
            return(false);
        }
        i++;
    }
    return(true);
}

// returns a ipv6 address string with 8 non empty fields
// expands double colon and ipv4 mapped 
function expandDoubleColonAndIpv4Mapped(ipv6str){
    var v4Arr=[];
    // take care of ipv4 mapped addresses such as ::ffff:192.0.2.47
    if (v4Arr=ipv6str.match(/:(\d+)\.(\d+)\.(\d+)\.(\d+)$/)){
        var t;
        var v4hex="";
        t=parseInt(v4Arr[1],10);
        v4hex+=t.toString(16);
        t=parseInt(v4Arr[2],10);
        v4hex+=t.toString(16);
        v4hex+=":";
        t=parseInt(v4Arr[3],10);
        v4hex+=t.toString(16);
        t=parseInt(v4Arr[4],10);
        v4hex+=t.toString(16);
        ipv6str=ipv6str.replace(/:(\d+)\.(\d+)\.(\d+)\.(\d+)$/,":"+v4hex);
    }
    if (ipv6str.match(/::/g)){
        // handle leading or trailing empty fields
        ipv6str=ipv6str.replace(/^::/,"0::");
        ipv6str=ipv6str.replace(/::$/,"::0");
        var ipFieldArr = ipv6str.split(":");
        var numOfEmptyFields=9-ipFieldArr.length;
        var i=0;
        var spliceStr=":";
        while(i<numOfEmptyFields){
            spliceStr+="0:";    
            i++;
        }
        ipv6str=ipv6str.replace(/::/,spliceStr);
    }
    return(ipv6str);
}

// returns a ipv6 address string with 8 non empty fields
function expandDoubleColon(ipv6str){
    if (ipv6str.match(/::/g)){
        // handle leading or trailing empty fields
        ipv6str=ipv6str.replace(/^::/,"0::");
        ipv6str=ipv6str.replace(/::$/,"::0");
        var ipFieldArr = ipv6str.split(":");
        var numOfEmptyFields=9-ipFieldArr.length;
        var i=0;
        var spliceStr=":";
        while(i<numOfEmptyFields){
            spliceStr+="0:";    
            i++;
        }
        ipv6str=ipv6str.replace(/::/,spliceStr);
    }
    return(ipv6str);
}

// exend the string object with a left padding function
String.prototype.h_lpadd = function (numberofdigits) {
    str=this;
    while(str.length < numberofdigits){
        str= "0" + str;
    }
    return(str);
}


// ipv6str must not contain a double colon
function expandFieldsTo4Digits(ipv6str){
    var ipFieldArr = ipv6str.split(":");
    var i=0;
    while(i<ipFieldArr.length){
        ipFieldArr[i]=ipFieldArr[i].h_lpadd(4);
        i++;
    }
    return(ipFieldArr.join(":"));
}

// compress 2018:04f2:0:0:0:0:0:0 to 2018:04f2::
function compressConsecutiveZeroToDoubleColon(ipv6str){
    // double colon is only allowed once in the string
    if (! ipv6str.match(/::/g)){
        ipv6str=ipv6str.replace(/:[0:]*:/,'::');
    }
    // if there is a zero field at the end then we can remove it too
    ipv6str=ipv6str.replace(/::0+$/,'::');
    // if there is a zero field at the start then we can remove it too
    ipv6str=ipv6str.replace(/^0+::/,'::');
    // now we can still have some thing like 1:2:0::3
    ipv6str=ipv6str.replace(/:0+::/,'::');
    ipv6str=ipv6str.replace(/::0+:/,'::');
    return(ipv6str);
}

// remove any double zeros e.g 0000:001::2 becomes 0:1::2
function stripLeadingZeros(ipv6str){
    ipv6str=ipv6str.replace(/^0+([^:])/,'$1');
    ipv6str=ipv6str.replace(/:0+([^:])/g,':$1');
    return(ipv6str);
}

// pad a mac address string with zeros to have always 2 digits, 2:4:f2:b1:a8:7f becomes 02:04:f2:b1:a8:7f
function mac2twodigits(macstr){
    var macArr = macstr.split(":");
    for (i=0;i<macArr.length;i++){
        macArr[i]=macArr[i].h_lpadd(2);
    }
    return(macArr.join(":"));
}

// the mac should have two digits in each position
//
//  take the mac address: for example 52:74:f2:b1:a8:7f
//  throw ff:fe in the middle: 52:74:f2:ff:fe:b1:a8:7f
//  reformat to IPv6 notation 5274:f2ff:feb1:a87f
//  convert the first octet from hexadecimal to binary: 52 = 01010010
//  invert the bit at position 2 (counting from 0):          01010000
//  convert octet back to hexadecimal: 01010000 = 50
//  replace first octet with newly calculated one: 5074:f2ff:feb1:a87f
//  prepend the link-local prefix: fe80::5074:f2ff:feb1:a87f
function mac2ipv6(macstr){
    var macArr = macstr.split(":");
    var invOct=0;
    macArr.splice(3, 0, "ff");
    macArr.splice(4, 0, "fe");
    invOct=parseInt(macArr[0],16) ^ 2;
    macArr[0]=invOct.toString(16);
    return("fe80::"+macArr[0]+macArr[1]+":"+macArr[2]+macArr[3]+":"+macArr[4]+macArr[5]+":"+macArr[6]+macArr[7]);
}

// convert a link local ipv6 string (fe80::5074:f2ff:feb1:a87f or fe80:0:0:0:5074:f2ff:feb1:a87f) to a mac
function ipv62mac(ipv6str){
    var expandedV6str;
    var matchArr;
    var resultstr="Addr. not link local format";
    expandedV6str=expandDoubleColon(stripLeadingZeros(ipv6str));
    if (! expandedV6str.match(/^fe80:0:0:0:/)){
        return(resultstr);
    }
    var zeropadded=expandFieldsTo4Digits(expandedV6str);
    matchArr=zeropadded.match(/:(\w\w)(\w\w):(\w\w)ff:fe(\w\w):(\w\w)(\w\w)$/);
    if (matchArr){
        var invOct=parseInt(matchArr[1],16) ^ 2;
        matchArr[1]=invOct.toString(16);
        matchArr[1]=matchArr[1].h_lpadd(2);
        var macArr=matchArr.slice(1,7);
        resultstr=macArr.join(":");
    }
    return(resultstr);
}

// convert /mask to ipv6 prefix mask (e.g /32 is ffff:ffff::)
// you have to ensure that mask is between 1 and 128 (inclusive)
// the return ipv6 prefix is a string.
function mask2prefix(maskstr){
    var m=parseInt(maskstr);
    var fullffff=Math.floor(m/16);
    var parialffff= m % 16;
    var i;
    var resultprefix=[];
    //initialize:
    for (i=0;i<8;i++){
        resultprefix[i]="0";
    }
    // fill full set:
    for (i=0;i<fullffff;i++){
        resultprefix[i]="ffff";
    }
    var nexti=i;
    if (parialffff>0){
        // fill parital set:
        var bitmask=0xffff0000;
        for (i=0;i<parialffff;i++){
            bitmask=bitmask>>1;
        }
        bitmask=bitmask& 0xffff;
        resultprefix[nexti]=bitmask.toString(16);
    }
    return(resultprefix.join(":"));
}
    
// calculates the start of the address range by doing a binary AND between ip and mask
function calcStartRange(ipv6str,maskstr){
    var m=parseInt(maskstr);
    var fullffff=Math.floor(m/16);
    var parialffff= m % 16;
    var i;
    var resultArr=[];
    ipv6str=expandDoubleColonAndIpv4Mapped(ipv6str);
    var ipv6strArr=ipv6str.split(":");
    if (ipv6strArr.length<8){   
        return("error: ipv6str has less than 8 fields");
    }
    //initialize result:
    for (i=0;i<8;i++){
        resultArr[i]="0";
    }
    // copy full set:
    for (i=0;i<fullffff;i++){
        resultArr[i]=ipv6strArr[i];
    }
    var nexti=i;
    if (parialffff>0){
        // fill parital set:
        var bitmask=0xffff0000;
        for (i=0;i<parialffff;i++){
            bitmask=bitmask>>1;
        }
        bitmask=bitmask& 0xffff;
        var tmp=bitmask & parseInt(ipv6strArr[nexti],16);
        resultArr[nexti]=tmp.toString(16);
    }
    return(stripLeadingZeros(resultArr.join(":")));
}

// calculates the end of the address range= IP with all bits that are zero in the prefix mask set to one
function calcEndRange(ipv6str,maskstr){
    var m=parseInt(maskstr);
    var fullffff=Math.floor(m/16);
    var parialffff= m % 16;
    var i;
    var resultArr=[];
    ipv6str=expandDoubleColonAndIpv4Mapped(ipv6str);
    var ipv6strArr=ipv6str.split(":");
    if (ipv6strArr.length<8){   
        return("error: ipv6str has less than 8 fields");
    }
    //initialize result:
    for (i=0;i<8;i++){
        resultArr[i]="ffff";
    }
    // copy full set:
    for (i=0;i<fullffff;i++){
        resultArr[i]=ipv6strArr[i];
    }
    var nexti=i;
    if (parialffff>0){
        // fill parital set:
        var bitmask=0xffff0000;
        for (i=0;i<parialffff;i++){
            bitmask=bitmask>>1;
        }
        bitmask=bitmask& 0xffff;
        var bitmaskinvert=bitmask^0xffff;
        var tmp=bitmask & parseInt(ipv6strArr[nexti],16);
        tmp=tmp | bitmaskinvert;
        resultArr[nexti]=tmp.toString(16);
    }
    return(stripLeadingZeros(resultArr.join(":")));
}
//===========big number math=====================
// bignum is an array datatype with 5 digits in each array field.
// 1234567 would be [34567,12]

// this function takes a string to represent an integer number (e.g "1234567") and
// converts it to the internal bigArr datatype. note: bigArr contains integer numbers (not strings).
function numstr2bigArr(intnumstr) {
    var bigArr=[];
    var i=0;
    while(intnumstr.length > 0 && i<1000){
        bigArr[i]=parseInt(intnumstr.slice(-5));
        intnumstr=intnumstr.slice(0,-5);
        i++;
    }
    return(bigArr);
}

// convert bigArr datatype to a display string
function bigArr2str(bigArr) {
    var a;
    a=bigArr.reverse();
    return(bigArr.join(""));
}

// multiply a bigArr by a small integer 
function bigmul(bigArr,smallint) {
    var i=0;
    var prod,car;
    car=0;
    for(i=0;i<bigArr.length;i++){
        prod=bigArr[i]*smallint + car;
        car = Math.floor( prod * (1/100000));
        prod= prod - (car * 100000);
        bigArr[i]= prod;
    }
    if (car > 0){
        bigArr[i]=car;
    }
    return(bigArr);
}

//=======end big number math=====================
